function [qme,qde,qe,rhse,dum3]=ele(ielem,coord,top,mat,pos,sol,soln,dum1,dum2);
%
% linear elastic element
%
% Parameters:
%
%	qme		: element mass matrix (not used)
%	qde		: element damping matrix (not used)
%  qe       : element stiffness matrix
%  rhse     : element right hand side
%  coord    : coordinates of all nodes
%  top      : topology array
%  ielem    : element number
%  mat      : material properties
%  				e   = mat(imat,1);   (Young's modulus)
%					nu	 = mat(imat,2);	(Poisson's ratio)
%  				axi = mat(imat,3);	(0: plane strain, 1: axi-symmetric, 2: plane stress)
%					fx  = mat(imat,4);   (Volume force in x-direction)
%					fy  = mat(imat,5);	(Volume force in y-direction)
%              eltype = mat(imat,11)(Element type specification, see ele_i)
%
% Remark
%  mat.types : 'ele;

qme=[]; qde=[]; dum3=[];

% get the number of nodes on this elemen
[nnodes,nsd,nelem,maxnlnodes,iimat,iitype] = sysinfo(size(coord),size(top));
nlnodes=length(nonzeros(top(ielem,1:maxnlnodes)));

% get the coordinates of the nodes of this elemen
nodcoord=coord(top(ielem,1:nlnodes),:);

% pick up some material parameter
imat=top(ielem,nlnodes+1);
e   = mat(imat,1);
nu  = mat(imat,2);
axi = mat(imat,3);
fx  = mat(imat,4);
fy  = mat(imat,5);

% element topology
etop     = top(ielem,:);

% position of the local degrees of freedom in the global solution array
[vpos,vshp]             = ele_i(1,[],etop,mat,5);
[n,dn,intworg,intcoord] = ele_s([],[],etop,mat,vshp(1,:));

% number of dofs for the displacements
nedofu = length(nonzeros(vpos(1:2,:)));

% number of integration points
nint   = length(intworg);

% extract the location of the displacement degrees of freedom
iu             = zeros(1,nedofu);
iu(1:2:nedofu) = nonzeros(vpos(1,:));
iu(2:2:nedofu) = nonzeros(vpos(2,:));

% initialize qe and rhs
qe=zeros(nedofu,nedofu);
rhse=zeros(nedofu,1);

% compute shear modulus and compression modulus from E and Nu
g    =e/(2*(1+nu));
kappa=e/(3*(1-2*nu));

if axi<2, % plane strain or axi-symmetric
   % D matrix currently assumes plane strain
   d= 1/3*g*[ 
      4     -2     0    -2;
      -2     4     0    -2;
      0      0     3  	 0;
      -2	   -2		 0		 4];
   
   d=d+kappa*[ 
      1 1 0 1;
      1 1 0 1;
      0 0 0 0;
      1 1 0 1];
   
elseif axi==2, % plane stress
   
   f0=(-2*g/3+kappa)^2/(4*g/3+kappa);
   f1=(4*g/3+kappa)-f0;
   f2=(-2*g/3+kappa)-f0;
   d=[f1 f2 0 0;
      f2 f1 0 0;
      0  0  g 0;
      0  0  0 0];
end

% initialize strain-displacement matrix
b=zeros(4,nedofu);
ii=0:2:nedofu-2;

% perform the element integration
for int=1:nint
   
   % compute x (=r)-coordinate for axi-symmetric case
   x=n(int,:)*nodcoord(:,1);
   if (axi==1),
      intw=2*pi*x*intworg;
   else
      intw=intworg;
   end
  
   %* get shape function derivatives with respect to global coordinates
   [dndxy,detj]=getderiv(dn,nodcoord,int);
   
   intwdetj=intw(int)*detj;
   
   %* compute the strain displacement matrix
   b(1,ii+1)=dndxy(:,1)';
   b(2,ii+2)=dndxy(:,2)';
   b(3,ii+1)=dndxy(:,2)';
   b(3,ii+2)=dndxy(:,1)';
   if axi==1,
      b(4,ii+1)=n(int,:)/x;
   end
  
   qe = qe + b'*d*b*intwdetj;
      
end

