function [par1,par2]=ele_i(ielem,coord,top,mat,ichois);
%
% description:
%
%   This class of elements solves either the incompressible Stokes problem
%   or a (in) compressible elasticity problem
%
%   paramters in the mat array
%     mat( 1) = g      : viscosity or Young's modulus
%     mat( 2) = kappa  : compression modulus if 0, incompressibility is assumed
%     mat( 3) = axi    : 0 for plane, and 1 for axi-symmetric problems
%     mat(11) = ietype : element type, see below
%    
% ietype:
%
% 1 - linear triangular element
% 2 - quadratic triangular element
% 3 - bi-linear quadrilateral
% 4 - bi-quadratic quadrilateral element

[nnodes,nsd,nelem,maxnlnodes,iimat,iitype]=sysinfo(size(coord),size(top));

imat     = top(ielem,iimat);
ietype   = mat(imat,11);
nlnodes  = length(nonzeros(top(ielem,1:maxnlnodes)));

par1=[]; par2=[];


if (ichois==1),
   
   if (ietype==1),
      par1=[2 2 2]; 					% Linear triangular-element
   elseif (ietype==2), 	
      par1=[2 2 2 2 2 2];			% Quadratic element
   elseif (ietype==3), 	
      par1=[2 2 2 2 ];	 			% bilinear element
   elseif (ietype==4),
      par1=[2 2 2 2 2 2 2 2 2];	% Bi-quadratic element
   end
   
   npar=length(par1);
   if npar~=nlnodes,
      string_err = ['Number of nodes = ' num2str(nlnodes) ...
            ' for element type ' num2str(ietype) ...
            ' in ielem ' num2str(ielem) ', should be '];
      error([string_err num2str(npar)]);
   end
   
   
elseif (ichois==2),
   
   if (ietype==1),
      par1=8*ones(1,nlnodes);
   elseif ietype==2,
      par1=8*ones(1,nlnodes);
   elseif ietype==3,
      par1=8*ones(1,nlnodes);
   elseif ietype==4,
      par1=8*ones(1,nlnodes);
   end
   
   
elseif (ichois==3),
   
   if (ietype==1),
      par1=[1 2; 2 3; 3 1];
   elseif (ietype==2),
      par1=[1 2; 2 3; 3 4; 4 5; 5 6; 6 1];
   elseif (ietype==3),
      par1=[1 2; 2 3; 3 4; 4 1];
   elseif (ietype==4),
      par1=[1 2; 2 3; 3 4; 4 5; 5 6; 6 7; 7 8; 8 1];
   end
   
elseif (ichois==4),
   
   
   if (ietype==1),
      % linear triangle
      gshp = [23 1 2];
      
   elseif (ietype==2 ),
      % 6-noded triangle 
      gshp(1,:)   = [26 2 7];
      
   elseif (ietype==3),
      % 4-noded quad
      gshp(1,:) = [14 1 3];
      
   elseif (ietype==4),
      % 9-noded quad
      gshp(1,:) = [19 1 3];
      
   end
   par1=gshp;
   
   
   
elseif (ichois==5),
   
   if (ietype==1),
      % 3-noded triangle with linear pressure
      vpos        = zeros(2,3);
      vpos(1,:)   = 1:2:5;
      vpos(2,:)   = vpos(1,:)+1;
      
      vshp      = zeros(2,3);
      vshp(1,:) = [23 2 3];
      vshp(2,:) = [23 2 3];
      
   elseif (ietype==2),
      % 6-noded triangle
      vpos        = zeros(3,nlnodes);
      vpos(1,:)   = 1:2:nlnodes*2;
      vpos(2,:)   = vpos(1,:)+1;
      
      vshp        = zeros(2,3);
      vshp(1,:)   = [26 2 7];
      vshp(2,:)   = [26 2 7];
      
   elseif (ietype==3),
      % 4-noded quad
      vpos        = zeros(2,nlnodes);
      vpos(1,:)   = 1:2:nlnodes*2;
      vpos(2,:)   = vpos(1,:)+1;
      
      vshp        = zeros(2,3);
      vshp(1,:)   = [14 1 2];
      vshp(2,:)   = [14 1 2];
      
   elseif (ietype==4),
      % 9-noded quad
      vpos        = zeros(2,nlnodes);
      vpos(1,:)   = 1:2:nlnodes*2;
      vpos(2,:)   = vpos(1,:)+1;
      
      vshp        = zeros(2,3);
      vshp(1,:)   = [19 1 3];
      vshp(2,:)   = [19 1 3];
      
   end
   par1=vpos; par2=vshp;
   
elseif ichois==6,
   
   if ietype==1,
      
      par1=[1 0; 0 1; 0 0];
      
   elseif ietype==2,
      
      par1=[1 0; 1/2 1/2; 0 1; 0 1/2; 0 0; 1/2 0];
      
   elseif ietype==3,
      
      par1=[-1 -1; 1 -1; 1 1; -1 1;];
      
   elseif ietype==4,
      
      par1=[-1 -1; 0 -1; 1 -1; 1 0; 1 1; 0 1; -1 1; -1 0; 0 0];
      
   end
   
elseif ichois==7,
   
   if ietype==1 | ietype==2 | ietype==3 | ietype==4,
      
      nvar=8; nmode=1;
      
      if ietype==1, nint=3; end
      if ietype==2, nint=7; end
      if ietype==3, nint=4; end
      if ietype==4, nint=9; end
      
   end
   par1=[nint nvar nmode];      
      
else
 
   error([' Ichois = ' num2str(ichois) ' not supported ']);

end
   
      


