function [top,coord,usercurves,userpoints,usersurfaces,topbnd]=...
   crmesh(curves,subarea,points,norder,itype,ichois);
%
% Purpose:
%
% creates a mesh for either regular or spectral elements (norder>2)
%
%
% Calling sequence:
%
% [top,coord,usercurves,userpoints,usersurfaces,topbnd]=crmesh(curves,subarea,points,norder);
% [top,coord,usercurves,userpoints,usersurfaces,topbnd]=crmesh(curves,subarea,points,norder,itype);
%
%
% Notes:
% 
% required data:
%
%  curves : pnt1 pnt2 nelem ibias bias          
%  subarea: crv1 crv2 crv3 crv4 imat
%  points : [x y] of point i
%  norder : order of the element 
%  itype  : 1: quadtrilateral, (default)
%           20: regular triangular element (linear of quadratic, depending on norder)
%           21: extended traingular element
%
% Examples:
%  points=[0 0; 1 0;; 1 2; 0 1];
%  curves=[1 2 2 1 .2;
%          2 3 3 1 .2;
%          3 4 2 1 .2;
%          4 1 3 1 .2];
%  subarea=[ 1 2 3 4 1];
%  norder=1;
%  [top,coord,usercurves,userpoints,usersurfaces,topbnd]=crmesh(curves,subarea,points,norder);
%  plmesh(coord,top);
%  plnodes(coord)
%  plnodnum(coord)
%
%  points=[0 0; 1 0; 2 0; 0 1; 1 1; 2 3];
%  curves=[1 2 2 1 .5
%          2 3 3 1 .5
%          3 6 4 1 .8
%          6 5 3 1 .5
%          5 4 2 1 .8
%          4 1 4 1 .5
%          5 2 4 1 .5];
%  subarea=[ 1 -7  5  6  1
%            2  3  4  7  2];
%  [top,coord,usercurves,userpoints,usersurfaces,topbnd]=crmesh(curves,subarea,points,norder);

if nargin<5, itype=1; end
if nargin<6, ichois=1; end

narea=length(subarea(:,1));

nlnodes=length(subarea(1,:))-1;
if (nlnodes~=4), error('initial topology not ok, must be 4-noded quads'); end

knode=0;
ielem=0;
top=[]; coord=[];


for iarea=1:narea
   
   crvs=subarea(iarea,1:4);
   
   if (curves(abs(crvs(1)),3)~=curves(abs(crvs(3)),3)),
      error(['number of elements on curve 1 and 3 of area ' num2str(iarea)...
            ' do not match']);
   end
   if (curves(abs(crvs(2)),3)~=curves(abs(crvs(4)),3)),
      error(['number of elements on curve 2 and 4 of area ' num2str(iarea)...
            ' do not match']);
   end
   
   
   %    if (curves(abs(crvs(1)),4)~=curves(abs(crvs(3)),4) | ...
   %        curves(abs(crvs(1)),5)~=curves(abs(crvs(3)),5)),
   %       error(['biasing of opposed curves on area ' num2str(iarea) ...
   %              ' do not match']);
   %    end
   
   %    if (curves(abs(crvs(2)),4)~=curves(abs(crvs(4)),4) | ...
   %        curves(abs(crvs(2)),5)~=curves(abs(crvs(4)),5)),
   %       error(['biasing of opposed curves on area ' num2str(iarea) ...
   %              ' do not match']);
   %    end
   
   
   % first curve
   if (crvs(1)>0),
      nodes(1)=curves(abs(crvs(1)),1);
      nodes(2)=curves(abs(crvs(1)),2);
   else
      nodes(1)=curves(abs(crvs(1)),2);
      nodes(2)=curves(abs(crvs(1)),1);
   end
   
   % second curve
   if (crvs(2)>0),
      nodes1=curves(abs(crvs(2)),1);
      nodes2=curves(abs(crvs(2)),2);
      if (nodes1~=nodes(2)),
         error(['subarea' num2str(iarea) ' improperly numbered']);
      end
      nodes(3)=nodes2;
   else
      nodes1=curves(abs(crvs(2)),2);
      nodes2=curves(abs(crvs(2)),1);
      if (nodes1~=nodes(2)),
         error(['subarea' num2str(iarea) ' improperly numbered']);
      end
      nodes(3)=nodes2;
   end
   
   % third curve
   
   if (crvs(3)>0),
      nodes1=curves(abs(crvs(3)),1);
      nodes2=curves(abs(crvs(3)),2);
      if (nodes1~=nodes(3)),
         error(['subarea' num2str(iarea) ' improperly numbered']);
      end
      nodes(4)=nodes2;
   else
      nodes1=curves(abs(crvs(3)),2);
      nodes2=curves(abs(crvs(3)),1);
      if (nodes1~=nodes(3)),
         error(['subarea' num2str(iarea) ' improperly numbered']);
      end
      nodes(4)=nodes2;
   end
   
   
   % fourth curve
   
   if (crvs(4)>0),
      nodes1=curves(abs(crvs(4)),1);
      nodes2=curves(abs(crvs(4)),2);
      if (nodes1~=nodes(4)),
         error(['subarea' num2str(iarea) ' improperly numbered']);
      end
      if (nodes2~=nodes(1)),
         error(['subarea' num2str(iarea) ' improperly numbered']);
      end
   else
      nodes1=curves(abs(crvs(4)),2);
      nodes2=curves(abs(crvs(4)),1);
      if (nodes1~=nodes(4)),
         error(['subarea' num2str(iarea) ' improperly numbered']);
      end
      if (nodes2~=nodes(1)),
         error(['subarea' num2str(iarea) ' improperly numbered']);
      end
   end
   
   
   
   coorde=points(nodes,:);
   
   %xi=gaulob(norder);
   
   %          1    2    3    4    5    6 7     8    9 10    11
   % subarea: pnt1 pnt2 pnt3 pnt4 imat n ibias bias m ibias bias
   
   n=curves(abs(crvs(1)),3);
   m=curves(abs(crvs(2)),3);
   
   signcrvs=sign(crvs);
   ibiasx1=curves(abs(crvs(1)),4);
   biasx1 =signcrvs(1)*curves(abs(crvs(1)),5);
   lx1=sqrt((coorde(2,1)-coorde(1,1))^2+(coorde(2,2)-coorde(1,2))^2);
   
   ibiasx2=curves(abs(crvs(3)),4);
   biasx2 =-signcrvs(3)*curves(abs(crvs(3)),5);
   lx2=sqrt((coorde(4,1)-coorde(3,1))^2+(coorde(4,2)-coorde(3,2))^2);
   
   ibiasy1=curves(abs(crvs(2)),4);
   biasy1 =signcrvs(2)*curves(abs(crvs(2)),5);
   ly1=sqrt((coorde(3,1)-coorde(2,1))^2+(coorde(3,2)-coorde(2,2))^2);
   
   ibiasy2=curves(abs(crvs(4)),4);
   biasy2 =-signcrvs(4)*curves(abs(crvs(4)),5);
   ly2=sqrt((coorde(4,1)-coorde(1,1))^2+(coorde(4,2)-coorde(1,2))^2);
   
   if (ibiasx1==5), biasx1=2*biasx1/lx1; end
   if (ibiasx2==5), biasx2=2*biasx2/lx2; end
   if (ibiasy1==5), biasy1=2*biasy1/ly1; end
   if (ibiasy2==5), biasy2=2*biasy2/ly2; end
   
   [xi1]=innmdxdy(norder*n,biasx1,2,-1,ibiasx1,0);
   [xi2]=innmdxdy(norder*n,biasx2,2,-1,ibiasx2,0);
   [xj1]=innmdxdy(norder*m,biasy1,2,-1,ibiasy1,0);
   [xj2]=innmdxdy(norder*m,biasy2,2,-1,ibiasy2,0);
   
   nn=norder*n;
   mm=norder*m;
   for in=1:n
      for jm=1:m
         ielem=ielem+1;
         
         k=0;
         for i=1:norder+1
            ii=(in-1)*norder+i;
            for j=1:norder+1
               jj=(jm-1)*norder+j;
               knode=knode+1;
               xi=(1-(jj-1)/mm)*xi1(ii)+(jj-1)/mm*xi2(ii);
               xj=(1-(ii-1)/nn)*xj2(jj)+(ii-1)/nn*xj1(jj);
               %                    shape=shape44([xi(ii) xj(jj)]);
               shapen=shape([xi xj],14);
               coord(knode,1)=shapen*coorde(:,1);
               coord(knode,2)=shapen*coorde(:,2);
               k=k+1;
               top(ielem,k)=knode;
               %if (iarea==1), in,jm,xi,xj,coord(knode,:),keyboard, end
            end
         end
         top(ielem,k+1)=subarea(iarea,5);
         
         
         if (norder<=2),
            if (norder==1),
               a=coord(top(ielem,2),:)-coord(top(ielem,1),:);
               b=coord(top(ielem,4),:)-coord(top(ielem,2),:);
               c=cross([a 0],[b 0]);
               if (c(3)>0),
                  iren=[1 2 4 3];
               else
                  iren=[1 3 4 2];
               end
               top(ielem,1:4)=top(ielem,iren);
            elseif (norder==2),
               a=coord(top(ielem,3),:)-coord(top(ielem,1),:);
               b=coord(top(ielem,9),:)-coord(top(ielem,3),:);
               c=cross([a 0],[b 0]);
               if (c(3)>0),
                  iren=[1 2 3 6 9 8 7 4 5];
               else
                  iren=[1 4 7 8 9 6 3 2 5];
               end
               
               top(ielem,1:9)=top(ielem,iren);
            end
         end
      end
   end
   % keyboard
end


% check for coincident nodes

epsdist=1e-6;
nnodes=length(coord(:,1));
nodes=1:nnodes;
nn=nnodes;
i=0;
while (i<nn),
   
   i=i+1;
   ii=1;
   while (isempty(ii)==0),
      dist=sqrt((coord(i+1:nn,1)-coord(i,1)).^2+...
         (coord(i+1:nn,2)-coord(i,2)).^2);
      ii=find(dist<=epsdist);
      nii=length(ii);
      if (nii~=0),
         for j=1:1
            jnode=i+ii(j);
            coord=[coord(1:jnode-1,:); coord(jnode+1:nn,:)];
            jj=(find(nodes==jnode));
            
            nodes(jj)=i;
            kk=find(nodes>=jnode);
            if (isempty(kk)==0), nodes(kk)=nodes(kk)-1; end
            nn=nn-1;
         end
      end
   end
end



[n,m]=size(top);
tophelp=reshape(top(:,1:m-1),n*(m-1),1);
tophelp=nodes(tophelp);
top(:,1:m-1)=reshape(tophelp,n,m-1);



if (ichois==1),
   origin=[0;0];
   dist=sqrt((coord(:,1)-origin(1)).^2+(coord(:,2)-origin(2)).^2);
elseif (ichois==2),
   dist=sqrt((coord(:,1)).^2);
elseif ichois==3,
   dist=sqrt((coord(:,2)).^2);
elseif ichois==4,
   dist=1:length(coord(:,1));
end




% index(i)=j, i=new node number,
%             j=old node number

[dist,index]=sort(dist);
coord=coord(index,:);

[n,m0]=size(top);
m=m0-1;
topp=reshape(top(:,1:m),n*m,1);
ihelp=zeros(n*m,1);
for i=1:n*m
   ihelp(i)=find(index==topp(i));
end

top(:,1:m)=reshape(ihelp,n,m);


% create bound

nbnd=length(curves(:,1));
for i=1:nbnd
   ipnt=curves(i,1);
   jpnt=curves(i,2);
   bound(i,:)=[points(ipnt,1) points(jpnt,1) points(ipnt,2) points(jpnt,2)];
end

nlnodes=length(top(1,:))-1;
nelem=length(top(:,1));
if nlnodes==9,
   for ielem=1:nelem
      nodes=top(ielem,1:9);
      coord(nodes(2),:)=(coord(nodes(1),:)+coord(nodes(3),:))/2;
      coord(nodes(4),:)=(coord(nodes(3),:)+coord(nodes(5),:))/2;
      coord(nodes(6),:)=(coord(nodes(5),:)+coord(nodes(7),:))/2;
      coord(nodes(8),:)=(coord(nodes(7),:)+coord(nodes(1),:))/2;
      coord(nodes(9),:)=sum(coord(nodes([1 3 5 7]),:))/4;
   end
end

n=length(top(1,:));
top(:,n+1)=top(:,n);

usercurves=crmesh_usrcrv(coord,curves,points);

userpoints=crmesh_usrpnt(coord,points);


% add midpoint node if itype==2,
if itype==2,
   [coord,top]=crmesh_mid(coord,top);
end

%transform to triangular mesh if required

if itype>=20,
   if itype==20, 
      iopt=0;
   elseif itype==21,
      iopt=1;
   end
   origin=[0 0 ];
   topbnd=[];
   bndcon=[];
   [top,coord,toporg,coordorg,bndcon,index,topp,...
         usercurves,usercurvesorg,topbnd]=...
      trimesh(top,coord,ichois,bndcon,origin,usercurves,userpoints,topbnd,iopt);
   
end


if norder==1, 
   ichois='lt';
elseif norder==2,
   ichois='qt';
end

if norder<=2,
   if norder==1 & itype<20,
      elgraph=[1 2; 2 3; 3 4; 4 1;];
   elseif norder==2 & itype<20,
      elgraph=[1 2; 2 3; 3 4; 4 5; 5 6; 6 7; 7 8; 8 1];
   elseif norder==1 & itype>=20,
      elgraph=[1 2; 2 3; 3 1];
   elseif norder==2 & itype>=20,
      elgraph=[1 2; 2 3; 3 4; 4 5; 5 6; 6 1];
   end
   
   pot=toppot(top);
   nodnod=transmesh_nodpot(coord,top,pot,elgraph);
   
   [usercurves,dum,usersurfaces,topbnd]=transmesh_crv(usercurves,coord,top,pot,nodnod,ichois);
end



function [usercurves]=crmesh_usrcrv(coord,curves,points);

ncrv=length(curves(:,1));

epsdist=1e-8;

for icrv=1:ncrv
   
   pnt=curves(icrv,1:2);
   x1=points(pnt(1),:);
   x2=points(pnt(2),:);
   t=(x2-x1);
   n=cross([0 0 1],[t 0]);
   
   a=[t(1) n(1); t(2) n(2)];
   
   b(1,:)=coord(:,1)'-x1(1);
   b(2,:)=coord(:,2)'-x1(2);
   
   c=a\b;
   
   jj=find(c(1,:)>-epsdist & c(1,:)<1+epsdist);
   ii=find(abs(c(2,jj))<epsdist);
   
   ii=jj(ii);
   
   nii=length(ii);
   
   % sequence the nodes to start pnt1
   dist=sqrt((x1(1)-coord(ii,1)).^2+(x1(2)-coord(ii,2)).^2);
   [y,k]=sort(dist);
   ii=ii(k);
   
   usercurves(icrv,1:nii)=ii;
   
end


function userpoints=crmesh_usrpnt(coord,points);

npoints=length(points(:,1));

for ipoint=1:npoints
   
   dist=sqrt((coord(:,1)-points(ipoint,1)).^2+(coord(:,2)-points(ipoint,2)).^2);
   
   [y,i]=min(dist);
   
   userpoints(ipoint)=i;
end


function [coord,top]=crmesh_mid(coord,top);

[nelem,m]=size(top);
if m~=6,
   error('size of top incorrect');
end

nnodes=length(coord(:,1));

top=[top(:,1:4) nnodes+(1:nelem)' top(:,5:6)];
coord=[coord;zeros(nelem,2)];

for ielem=1:nelem
   nodcoord=coord(top(ielem,1:4),:);
   coord(nnodes+ielem,:)=mean(nodcoord);
end

