%Script to solve one-dimensional convection-diffusion problems with mlfem_nac
%
% Definition of variables:
%                   nnodes: number of nodes
%                   nsd   : space dimension (1 or 2)
%                   nelem : number of elements
%                   maxnlnodes: maximum number of nodes in an element
%                   iimat: location of material type in TOP array
%                   iitype: location of element type in TOP array
%                   q:   the global stiffness matrix 
%                   rhs: the global right-hand-side column
%                   qk:  the global stiffness matrix
%                   qm:  the global mass matrix
%                   qd:  the global damping matrix
%
%   Last change:     Cees Oomens 14-5-2008 

% Determine some global variables by the function sysinfo
[nnodes,nsd,nelem,maxnlnodes,iimat,iitype] = sysinfo(size(coord),size(top));

% Compute the pos and dest array
[pos,dest]=equatnr(coord,top,mat);

% ndof: the total number of degrees of freedom
ndof = max(max(pos));

% Define number of unknows for each node
ldof=ones(1,maxnlnodes);

% initialize qk, qm, qd and rhs;
qk=spalloc(ndof,ndof,sum(ldof)^2);
qm=qk;
qd=qk;
rhs=zeros(ndof,1);

%Determine whether this program is time dependent or not
if istat==1,      % steady solution
   sol=zeros(ndof,1);
elseif istat==2,  % unsteady solution
   sol=zeros(ndof,ntime);   % sol(:,1) contains the initial condition
end

% Loop over elements to determine the element contributions to the global stiffness matrix
% and to assemble q and rhs
for ielem=1:nelem
     % compute the element stiffness matrix and right-hand-side array using elm1dcd (see elmlib)
    [qme,qde,qke,rhse]=elm1dcd(ielem,coord,top,mat.mat);
    
    % assemble the global stiffness matrix and the right-hand-side colomn
    ii=pos(ielem,:);
    qk(ii,ii)=qk(ii,ii)+qke;
    qm(ii,ii)=qm(ii,ii)+qme;
    qd(ii,ii)=qd(ii,ii)+qde;

    rhs(ii)=rhs(ii)+rhse;
end

if istat==1,
   % Solve the system of equations, stationairy problem
   q=qk+qd;
   sol=solvestat(q,rhs,bndcon,[],dest);
   
elseif istat==2,
   %  Solve the system of equations, instationairy problem
   q=qm/dt+theta*qd+theta*qk;
   soln=sol(:,1);
   
   for itime=1:ntime
      rhsn=rhs+(qm/dt-(1-theta)*(qd+qk))*soln;
      sol(:,itime)=solvestat(q,rhsn,bndcon,[],dest);
      soln=sol(:,itime);
   end
   
end
