function [xp,yp,xylevel,hl]=conoddat(noddat,coord,top,mat,nlevel,ivar,itime,option,tol)
%
%
% [xp,yp,xylevel]=conoddat(noddat,coord,top,mat);
% [xp,yp,xylevel]=conoddat(noddat,coord,top,mat,nlevel);
% [xp,yp,xylevel]=conoddat(noddat,coord,top,mat,nlevel,ivar);
% [xp,yp,xylevel]=conoddat(noddat,coord,top,mat,nlevel,ivar,itime);
% [xp,yp,xylevel]=conoddat(noddat,coord,top,mat,nlevel,ivar,itime,option);
% [xp,yp,xylevel]=conoddat(noddat,coord,top,mat,nlevel,ivar,itime,option,tol);
%
% algorithm:
%   - the element is divided into triangles
%   - the corners of each triangle is made up of the nodes defining elgraph and
%     the midpoint, having the average position of the nodes in elgraph.
%   - along each side of the triangle a linear interpolation is used
%   - the midpoint value is taken at the average of the nodes in elgraph
%
% input:    
%   noddat       : nodal data
%   coord        : nodal coordinates
%   top          : topology
%   mat          : material properties
%   nlevel  (10) : number of levels                         
%   ivar    (1)  : physical variable number to be plotted   
%   itime   (1)  : time level 
%   option  (1)  : options
%                  option(1)=1:  nlevels are drawn between minimum and maximum.
%                  option(1)=2:  nlevels are drawn between   0 and max and
%                                nlevels are drawn between min and 0
%                  option(1)=3:  nlevels are drawn between option(2) and 
%                                                          option(3)
%                  option(1)=4:  levels are specified in option(2:length(option))
%   tol    (1e-5): minima and maxima are offset by a relative tolerance of tol
%                  min+tol*min, max-tol*max;
%                  only if option=1 or 2
%
% if option(1)<0:  legend is not drawn
%                          
% output:
%   xp           : x-coordinates of line segments of contour lines
%   yp           : y-coordinates of line segments of contour lines
%   xylevel      : level values of line segments
%
% remark:
%   xp, yp and xylevel can be used as follows:
%
%   covalues(xp,yp,xylevel);
%
% this places contour line level values at specified positions
%
%      
% Modifications: 03-08-95 : clevel definition for option=2 corrected 
%                10-08-95 : offset of minima and maxima                    

if nargin<4,
   error('illegal number if input arguments, must be at least 4');
end
if nargin<5
   nlevel=10;
end
if nargin<6
   ivar=1;
end
if nargin<7
   itime=1;
end
if nargin<8
   option=1;
end
if nargin<9,
   tol=1e-5;
end



hf=gcf;

name=get(hf,'name');
number=get(hf,'numbertitle');
figure(hf);

set(hf,'name','processing, please wait','numbertitle','off'); pause(.1);

[nnodes,nsd,nelem,maxnlnodes,iimat,iitype] = sysinfo(size(coord),size(top));

sz=size(noddat);
if length(sz)<2, sz(2)=1; end
if length(sz)<3, sz(3)=1; end
if sz(1)~=nnodes | sz(2)<ivar | sz(3)<itime,
   error(['Incorrect size if noddat, or incorrect ivar or itime']);
end

data = noddat(:,ivar,itime);

xp=[0;0]; yp=[0;0]; xylevel=[0];

ipick=0;


signoption=sign(option(1));
option(1)=abs(option(1));

if option(1)==1,

  [cmax ,imax]=max(data);
  [cmin ,imin]=min(data);

  cmin=cmin+tol*cmin; 
  cmax=cmax-tol*cmax;

  dlevel=(cmax-cmin)/nlevel;
  clevel=cmin:dlevel:cmax;

elseif option(1)==2,
 
  clevel=[];
  [cmax, imax]=max(data);
  [cmin ,imin]=min(data);
  
  cmin=cmin+tol*cmin; 
  cmax=cmax-tol*cmax;

  if cmin<0,
     dlevel=cmin/nlevel;
     clevel=cmin:-dlevel:dlevel;
  end
  if cmax>0,
     dlevel=cmax/nlevel;
     clevel=[clevel 0:dlevel:cmax]; 
  end  
  
elseif option(1)==3,


  cmin=option(2);
  cmax=option(3);
  dlevel=(cmax-cmin)/nlevel;
  clevel=cmin:dlevel:cmax;
  
elseif option(1)==4,

  nopt=length(option);
  clevel=option(2:nopt);

else

  error('illegal option selected, must be 1, 2 or 3');
  
end
  
%colors = brighten(jet,.25);
colors = jet;
nlevel = length(clevel);
ncolor = length(colors(:,1));
dcolor = floor(ncolor/nlevel);
ccolor = flipud(colors(ncolor:-dcolor:1,:));

  

set(hf,'name','Identifying the edge','numbertitle','off'); pause(.1);

graphtop=top2grph(coord,top,mat);
gplot(graphtop-graphtop',coord,'-w'), axis off, axis image

hold on



% store data element wise
noddata=data;
data=zeros(nelem,maxnlnodes);
for inode=1:maxnlnodes
    nodes=top(:,inode);
    ii=find(nodes);
    data(ii,inode)=noddata(nodes(ii));
end


[xp,yp,xylevel,hl]=cntr(coord,top,mat,data,clevel,ccolor,hf,signoption);

set(hf,'name',name,'numbertitle',number)
