function [M] = mvnoddat(noddat,coord,top,mat,ivar,itime,idim,options);

% M=mvnoddat(noddat,coord,top,mat);
% M=mvnoddat(noddat,coord,top,mat,ivar,itime,idim);
% M=mvnoddat(noddat,coord,top,mat,ivar,itime,idim,options);
%
% plots nodal data on mesh defined by coord,top and mat
%
% Input:
%
%  NODDAT : nodal data, may be both in multidimimensional and oridinary form
%           if multidimensional:
%           MDGETMAT(NODDAT,INODE,IVAR,ITIME) = 
%           variable IVAR on node INODE on time ITIME
%           if ordinary:         
%           SCALDAT(INODE,IVAR) = variable IVAR on node INODE  
%  COORD  : nodal coordinates
%  TOP    : mesh topology 
%  MAT    : material/element parameter array
%  IVAR   : optional index of variable (default=1)
%  ITIME  : optional index of time step (default=1)
%  IDIM   : optional plot dimension choice parametes 
%           idim=2: 2 D plot
%           idim=3: 3 D plot
%           default IDIM = NSD (number of spatial dimensions)
%  options(1:3): aspectratio of the plot, if idim=3
%
% Output:
%  H      : handle of fill(3) (e.g. to set edgecolor set(h,'edgecolor','none')
%
% If you want to use a certain component ICOMP of SOL for the colour plot,
% plnoddat can be called according to
%
% plnoddat(sol(dest(:,icomp)),coord,top,mat);
 
% plot function

if nargin<4,
   error('Illegal number of input arguments, must be at least 5');
end
if nargin<5
   ivar  = 1;
end
if nargin<6
   itime  = 1;
end
if nargin<7
   idim  = 2;
end
if nargin<8,
   options=[];
end

if (idim~=3 & idim~=2), 
   idim=2;
end

[nnodes,nsd,nelem,maxnlnodes,iimat,iitype] = sysinfo(size(coord),size(top));


sz=size(noddat);
if length(sz)<2, sz(2)=1; end
if length(sz)<3, sz(3)=1; end
if sz(1)~=nnodes | sz(2)<ivar | sz(3)<itime,
   error(['Incorrect size if noddat, or incorrect ivar or itime']);
end

coord = [coord zeros(nnodes,3-nsd) ];

x = zeros(maxnlnodes,nelem);
y = zeros(maxnlnodes,nelem);
z = zeros(maxnlnodes,nelem);
c = zeros(maxnlnodes,nelem);
 
for ielem = 1:nelem

    connect = top(ielem,1:maxnlnodes);
    elgraph = eleminfo(ielem,coord,top,mat,3);
    if isempty(elgraph)
       nodes   = nonzeros(connect);
    else
       nodes   = connect(elgraph(:,1));     
    end
    nlnodes = length(nodes);

    %* fill dummy values for all possible local nodes
    x(:,ielem) = ones(maxnlnodes,1) * coord(nodes(1),1) ;
    y(:,ielem) = ones(maxnlnodes,1) * coord(nodes(1),2) ;

    %* fill real values for actual local nodes
    x(1:nlnodes,ielem) = coord(nodes,1);
    y(1:nlnodes,ielem) = coord(nodes,2);
    if nsd==3
       z(:,ielem) = ones(maxnlnodes,1) * coord(nodes(1),3);
       z(1:nlnodes,ielem) = coord(nodes,3);
    end

    %* fill color with nodal data
    %if mdcheck
    %   dat = mdgetmat(noddat,nodes,ivar,itime);
    %else
    %   dat = noddat(nodes,ivar);
    %end
    
    if length(ivar)>1,
       n=length(ivar);
       for i=1:n
          dat=squeeze(noddat(nodes,ivar(i),itime));
          c(:,ielem,i) = ones(maxnlnodes,1) * dat(1) ;
          c(1:nlnodes,ielem,i) = dat;
       end
    elseif length(itime)>1,
       n=length(itime);
       for i=1:n
          dat=squeeze(noddat(nodes,ivar,itime(i)));
          c(:,ielem,i) = ones(maxnlnodes,1) * dat(1) ;
          c(1:nlnodes,ielem,i) = dat;
       end
    else
       dat = squeeze(noddat(nodes,ivar,itime));
       c(:,ielem) = ones(maxnlnodes,1) * dat(1) ;
       c(1:nlnodes,ielem) = dat;
    end
    
    
    
 end;
 
 cmax=max(max(max(c)));
 cmin=min(min(min(c)));
 
    xlim=[min(coord(:,1)) max(coord(:,1))];
    ylim=[min(coord(:,2)) max(coord(:,2))];
    zlim=[cmin cmax];
    if length(options)<3,
       aratio=[1 norm(ylim)/norm(xlim) 1];
    else
       aratio=options(1:3);
    end
    
 for i=1:n
    if idim==2,
       h0=fill(x,y,c(:,:,i)/cmax); 
       axis('off'); axis('image'); caxis([cmin cmax]);
       set(h0,'edgecolor','white'); colormap('jet');
    elseif nsd==3
       h0=fill3(x,y,z,c(:,:,i)/cmax); grid; 
       axis('off'); axis('image'); caxis([cmin cmax])
       set(h0,'edgecolor','white'); colormap('jet');
    elseif idim==3,
       h0=fill3(x,y,c(:,:,i),c(:,:,i)/cmax); grid,% set(gca,'aspectratio',[1 1]);
       axis('on'); caxis([cmin cmax]); % axis('image');
       set(h0,'edgecolor','white'); colormap('jet');
    end
    
   set(gca,'xlim',xlim,'ylim',ylim,'zlim',zlim,'plotboxaspectratio',aratio)
     if i==1, M=moviein(n); end
   M(:,i)=getframe;
 end
 
 
 


if nargout>0
   h = h0;
end

colorbar



