function M = mvvec(sol,coord,top,dest,mat,scalein,original,idim,nodes,xybox)
%
% M=mvvec(sol,coord,top,dest,mat);
% M=mvvec(sol,coord,top,dest,mat,scale);
% M=mvvec(sol,coord,top,dest,mat,scale,original);
% M=mvvec(sol,coord,top,dest,mat,scale,original,idim);
% M=mvvec(sol,coord,top,dest,mat,scale,original,idim,nodes);
%
% makes a movie (M) of the vector plot of sol
%
% input:
%   sol          : displacements, may be either in solution type form or as
%                  nodal displacements
%   coord        : nodal coordinates
%   top          : mesh topology
%   dest         : destination array 
%   mat          : optional material/element parameter array
%   scale    (1) : optional scale factor (default = 1)
%                  'max': scale=1/max(sol);
%   original (1) : optional choice parameter, indicating whether the original mesh 
%                  must be drawn fully (2), boundary only (1) or not (0)
%                  if (original==-1), only deformed boundary is shown
%                  default = 1
%   idim    (nsd): optional plot dimension choice parametes 
%                  idim=2: 2 d plot
%                  idim=3: 3 d plot
%                  default idim = nsd (number of spatial dimensions)%
%
% output:
%	 M				  : Movie matrix
%


[nnodes,nsd] = size(coord);

%** check input / set defaults
if nargin<5
   mat=[];
end
if nargin<6
   scalein=1;
end
if nargin<7
   original = 1;
end
if nargin<8
   idim = nsd;
end

[nnodes,nsd,nelem,maxnlnodes,iimat,iitype]=sysinfo(size(coord),size(top));
if nargin<9,
   nodes=1:nnodes; 
end

if nargin<10,
   xybox=[];
end

if isempty(nodes),
   nodes=1:nnodes;
end



%** set parameters
color    = 'r';
colororg = 'y';

nldof        = length(dest(1,:));
if nsd>3  
   coord = coord(:,1:3); %* length of coord might be >nsd for plate elements
   nsd=3;
end
if idim>nsd
   coord = [coord zeros(nnodes,idim-nsd) ];
end

%** plot
if idim==2,
   a = top2grph(coord,top,mat);
   gplot(a-a',coord,color); axis('off'); axis('image');
else  
   a = top2grph(coord,top,mat)
end

xlim=[min(coord(:,1)) max(coord(:,1))]; dx=xlim(2)-xlim(1); xlim=xlim+[-dx dx]/10;
ylim=[min(coord(:,2)) max(coord(:,2))]; dy=ylim(2)-ylim(1); ylim=ylim+[-dy dy]/10;
d=min([dx dy])/5;

ntime=length(sol(1,:));
for itime=1:ntime
   
   clf
   %** if required, extract nodal displacements from solution vector
   [m,n] = size(sol(:,itime));
   if n==1;
      %** sol is solution type vector, extract nodal displacements
      noddisp    = sol2nod(sol(:,itime),dest(1:nnodes,:));
   elseif m==nnodes & n>=nsd
      %** sol is already in nodal form
      noddisp = sol;
   else
      error('illegal size of array sol')
   end
   
   %** use only components 1:idim (required for e.g. plate elements)
   if idim>size(noddisp,2)
      noddisp = [noddisp zeros(nnodes,idim-size(noddisp,2)) ];
   else
      noddisp    = noddisp(:,1:idim);
   end
   
   if strmatch(scalein,'max'),
      scale=d/(max(max(noddisp)));
   else
      scale=scalein;
   end
  
   %** compute displaced coordinates
   coorddisp = coord + scale*noddisp;
   
   %** set up plot data   
   x  = coord(nodes,1);
   dx = scale*noddisp(nodes,1);
   y  = coord(nodes,2);
   dy = scale*noddisp(nodes,2);
   if nsd==3
      z  = coord(nodes,3);
      dz = scale*vec(nodes,3);
   end 
   
   %** plot vectors
   if idim==2 
      hvec0=plarrow(x,y,dx,dy,0,'r');
      %arrow([x y],[x+dx y+dy]);
   else
      hvec0=plarrow3(x,y,z,dx,dy,dz,0,'r');
      %arrow([x y z],[x+dx y+dy z+dz]);
   end
   
   if isempty(xybox),   
      set(gca,'xlim',xlim,'ylim',ylim)
   else
      set(gca,'xlim',xybox(1,:),'ylim',xybox(2,:));
   end
   
   
   %axis('image')
   if itime==1, 
      M=moviein(n); 
   end
   M(:,itime)=getframe;
end

