function noddat = sol2nod(sol,arg2,top,mat,pos,vars,iopt)
%
% returns an array containing the solution in nodal points
%
% noddat = sol2nod(sol,dest);                    % option 1
% noddat = sol2nod(sol,coord,top,mat,pos);       % option 2
% noddat = sol2nod(sol,coord,top,mat,pos,vars);  % option 2.1
%
% option 1: Use dest to extract nodal values from sol
%           This option works well (and fast) for a constant number of
%           variables per node all having the same physical meaning
%           Alternative call: 
%
%              noddat = sol2nod(sol,dest(:,ivar));
%
%              This selects the i'th variable from sol, 
%              this variable must be at the i'th position at all nodes
%
% option 2: shape function of the physical variables are used
%           to compute values at all the element nodes of the
%           basic geometrical element.
%
%           This option works well (and slow) if a non-constant number of
%           variables (or discontinuous interpolations) is used
%           
% 
% Input:
%
%   SOL   : solution vector(s), size = NDOF x NSOL
%   DEST  : equation numbers per nodal point, size = NNODES x NLDOF
% 
%   TOP   : topology array
%   MAT   : material properties
%   POS   : position array (from equatnr)
%
% Output:
%
%    NODDAT = solution in nodal points, size NNODES x (NSOL*NLDOF)
%  
% NODDAT      : data in global nodal points, generally stored in 
%               multidimensional format
%
%               NODDAT(INODE,IVAR,ITIME) =
%               the value of variable IVAR in global node INODE
%               on timestep ITIME
%

if (nargin==2),
   dest=arg2;
   vars=[];
elseif (nargin==5),
   coord=arg2;
   vars=[];
   iopt=0;
elseif (nargin==6),
   coord=arg2;
   iopt=0;
elseif (nargin==7),
   coord=arg2;
else
   error('Illegal number of input arguments');
end

if (nargin==2),
    
   %** nodal solution can be adressed directly form DEST
   [nnodes,nldof] = size(dest);
   nsol           = length(sol(1,:));

   if nsol==1
      %** nodal solution can be stored as ordinary matrix
      noddat = zeros(nnodes,nsol*nldof);
      for ildof=1:nldof
          ii = find(dest(:,ildof));
          jj = (1:nldof:(nsol*nldof) )+ ildof - 1;
          noddat(ii,jj) = sol(dest(ii,ildof),:);
      end
    else
      %** nodal solution must be stored as multidimensional matrix
      %noddat = mdzeros(nnodes,nldof,nsol);
      noddat = zeros(nnodes,nlnodes,nsol);
      for ildof=1:nldof
          ii = find(dest(:,ildof));
          %noddat = mdputmat(noddat,(sol(dest(ii,ildof),:)),ii,ildof,1:nsol);
          noddat(ii,ildof,1:nsol) = sol(dest(ii,ildof),:);
      end
    end
          
else

   %** nodal solution can not be adressed directly. First element solutions
   %   must be computed. Then a weighted nodal averaging is applied
   [elsol,elweight] = sol2elm(coord,top,mat,pos,sol,vars,iopt);
   noddat           = elm2nod(elsol,elweight,top);

end

%** if possible, transform to ordinary matrix
%if ismd(noddat)<0
%      noddat = md2mat(noddat);
%end 
