function [n,dn]=shape(intcoord,itype)
%
% [n,dn]=shape(intcoord,itype);
%
% definition of shape functions for quadrilateral elements
%
% input:
%   intcoord  : ksi, eta, zeta coordinates of integration points
%   itype     : 1   -  constant interpolation
%
%             : 2   -  2-noded line element
%             : 3   -  3-noded line element
%
%             : 14  -  4-noded quad (default)
%             : 15  -  4-noded quad with bubble
%             : 18  -  8-noded quad
%             : 19  -  9-noded quad
%
%             : 23  -  3-noded triangle 
%             : 24  -  4-noded triangle (13-traingle with bubble function)
%             : 26  -  6 triangle
%             : 27  -  7-noded triangle (16-triangle with bubble function
%
%             : 38  -  8-noded brick
%             : 320 - 20-noded brick        
%
% output:
%   n         : shape functions
%                 n(1:nint,1:nlnodes)
%                    nint=number of integration points=length(intcoord(:,1));
%   dn        : shape function derivative with respect to ksi, eta and zeta)
%                 dn(1:nlnodes,1:nsd*nint)
%                    nlnodes = number of nodal points
%                    nsd     = space dimension = length(intcoord(1,:));


if (nargin==1), itype=4; end


nsd=length(intcoord(1,:));

if (itype==1),

   nint=length(intcoord(:,1));
   n=ones(nint,1);
   dn=zeros(1,nint*2);
   
elseif (itype==2),

   [n,dn]=shline2(intcoord);

elseif (itype==3),

   [n,dn]=shline3(intcoord);

elseif (itype==14),

   [n,dn]=shquad4(intcoord);

elseif (itype==15),

   [n,dn]=shquad5(intcoord);

elseif (itype==16),

   [n,dn]=shquad4p1(intcoord);

elseif (itype==18),

   [n,dn]=shquad8(intcoord);

elseif (itype==19),

   [n,dn]=shquad9(intcoord);
   
elseif (itype==23),

   [n,dn]=shtri3(intcoord);

elseif (itype==24),

   [n,dn]=shtri4(intcoord);

elseif (itype==26),

   [n,dn]=shtri6(intcoord);

elseif (itype==27),
   
   [n,dn]=shtri7(intcoord);
   
elseif (itype==38),

   [n,dn]=shbrick8(intcoord);
   
elseif (itype==320),

   [n,dn]=shbrck20(intcoord);
   

else
   error([' This shape function type = ' num2str(itype) ' is not supported']);

end


