function [n,dn,detj]=shtri7(intcoord,coord);
%
% [n,dn]=shtri7(intcoord);
% [n,dn,detj]=shtri7(intcoord,coord);
%
%
% shape function routine for 7 noded triangular element
%
% input:
%   intcoord :  local coordinates (intcoord,eta) of integration points
%   coord    :  nodal coordinates
%
% output:
%   n        :  shape functions at integration points
%   dn       :  derivative of shape functions with resp. to intcoord and eta
%   detj     :  det(jacobian)
%

% remark:
%   if coord is not specified, a unit triangular cel is chosen
%   this shape function routine must then be used in combination
%   with getderiv
%
%   otherwise: dn are the shape function derivatives with respect to the
%              global coordinates

if (nargin==1),
   coord=[  0   0; 
            .5  0; 
            1   0; 
            .5  .5; 
            0   1; 
            0   .5; 
            .33 .33];
end


nint    = length(intcoord(:,1));
nsd     = length(intcoord(1,:));
nlnodes = 7;
n       = zeros(nint,nlnodes);
dn      = zeros(nlnodes,nint*2);


if nsd~=3,
   error('This shape function only valid for 2D problems');
end


delta = (coord(5,1)-coord(3,1)) * (coord(1,2)-coord(3,2)) - ...
        (coord(3,2)-coord(5,2)) * (coord(3,1)-coord(1,1));

detj=delta/2;

dla1dx = (coord(3,2)-coord(5,2))/delta;
dla2dx = (coord(5,2)-coord(1,2))/delta;
dla3dx = (coord(1,2)-coord(3,2))/delta;

dla1dy = (coord(5,1)-coord(3,1))/delta;
dla2dy = (coord(1,1)-coord(5,1))/delta;
dla3dy = (coord(3,1)-coord(1,1))/delta;



for ip=1:nint

    k1 = intcoord(ip,1);
    k2 = intcoord(ip,2);
    k3 = intcoord(ip,3);

    n(ip,1) = k1*(2*k1-1)+3*k1*k2*k3;
    n(ip,2) = k2*(2*k2-1)+3*k1*k2*k3;
    n(ip,3) = k3*(2*k3-1)+3*k1*k2*k3;
    n(ip,4) = 4*k1*k2-12*k1*k2*k3;
    n(ip,5) = 4*k2*k3-12*k1*k2*k3;
    n(ip,6) = 4*k1*k3-12*k1*k2*k3;
    n(ip,7) = 27*k1*k2*k3;

    k = 2*ip-1;

    dn(1,k) = dla1dx*(2*k1-1)+k1*2*dla1dx +...
              3*dla1dx*k2*k3 + 3*k1*dla2dx*k3 + 3*k1*k2*dla3dx; 
    dn(2,k) = dla2dx*(2*k2-1)+k2*2*dla2dx +...
              3*dla1dx*k2*k3 + 3*k1*dla2dx*k3 + 3*k1*k2*dla3dx;
    dn(3,k) = dla3dx*(2*k3-1)+k3*2*dla3dx +...
              3*dla1dx*k2*k3 + 3*k1*dla2dx*k3 + 3*k1*k2*dla3dx;
    dn(4,k) = 4*dla1dx*k2+4*k1*dla2dx - ...
              (12*dla1dx*k2*k3 + 12*k1*dla2dx*k3 + 12*k1*k2*dla3dx);
    dn(5,k) = 4*dla2dx*k3+4*k2*dla3dx - ...
              (12*dla1dx*k2*k3 + 12*k1*dla2dx*k3 + 12*k1*k2*dla3dx);
    dn(6,k) = 4*dla1dx*k3+4*k1*dla3dx - ...
              (12*dla1dx*k2*k3 + 12*k1*dla2dx*k3 + 12*k1*k2*dla3dx);
    dn(7,k) = 27*dla1dx*k2*k3 + 27*k1*dla2dx*k3 + 27*k1*k2*dla3dx;


    dn(1,k+1)= dla1dy*(2*k1-1)+k1*2*dla1dy+...
               3*dla1dy*k2*k3 + 3*k1*dla2dy*k3 + 3*k1*k2*dla3dy; 
    dn(2,k+1)= dla2dy*(2*k2-1)+k2*2*dla2dy+...
               3*dla1dy*k2*k3 + 3*k1*dla2dy*k3 + 3*k1*k2*dla3dy;
    dn(3,k+1)= dla3dy*(2*k3-1)+k3*2*dla3dy+...
               3*dla1dy*k2*k3 + 3*k1*dla2dy*k3 + 3*k1*k2*dla3dy;
    dn(4,k+1)= 4*dla1dy*k2+4*k1*dla2dy - ...
               (12*dla1dy*k2*k3 + 12*k1*dla2dy*k3 + 12*k1*k2*dla3dy);
    dn(5,k+1)= 4*dla2dy*k3+4*k2*dla3dy - ...
               (12*dla1dy*k2*k3 + 12*k1*dla2dy*k3 + 12*k1*k2*dla3dy);
    dn(6,k+1)= 4*dla1dy*k3+4*k1*dla3dy - ...
               (12*dla1dy*k2*k3 + 12*k1*dla2dy*k3 + 12*k1*k2*dla3dy);
    dn(7,k+1)=27*dla1dy*k2*k3 + 27*k1*dla2dy*k3 + 27*k1*k2*dla3dy;



end


ii=[1 4 2 5 3 6 7];

n=n(:,ii);
dn=dn(ii,:);



