function [mass,damp,stif,rhs,elemdata_out] = ...
                   sysbuild(coord,top,mat,pos,dest, ...
                            options,sol,nodaldata,elemdata_in,elemparam)

%**************************************************************************
% [mass,damp,stif,rhs,elemdata_out] = ...
%              sysbuild(coord,top,mat,pos,dest, ...
%                       options,sol,nodaldata,elemdata_in,elemparam)
%
%  General system building function. Creates the global stiffness and mass
%  matrices and right hand side vector for static/dynamic, linear/nonlinear,
%  implicit/explicit problems depending of the options set in OPTIONS
% 
% input:
% 
% COORD        : nodal point coordinates
% TOP          : element topology
% MAT          : element/material properties
% POS          : global equation numbers per element 
% DEST         : global equation numbers per node
% OPTIONS      : option paramater array 
% SOL          : solution vector(s) from previous iteration/time steps  
% NODALDATA    : optional nodal data array, 
%                NODALDATA(INODE,:) = data for global node INODE. Definition
%                is problem/user dependent.
% ELEMDATA_IN  : optional element data array 
%                ELEMDATA_IN(IELEM,:) = data for element IELEM.  Definition
%                is problem/user dependent.
% ELEMPARAM    : optional parameter array. Definition
%                is problem/user dependent.
%
% output:
%
% MASS         : system mass matrix
% DAMP         : system damping matrix
% STIF         : system stiffness matrix
% RHS          : system right hand side vector (body forces)
%                for each degree of freedom, in each node, for each element.
% ELEMDATA_OUT : element ouyput data. 
%                ELEMDATA_IN(IELEM,:) = data for element IELEM.  Definition
%                is problem/user dependent.
%
% On input, COORD and TOP must be defined as
%
% COORD(INODE,ISD)  = X        : coordinate of node INODE in dimension ISD 
%
% TOP(IELEM,ILNODE) = INODE   : global node number INODE for each element IELEM
%                               and local node number ILNODE
% TOP(IELEM,ILNODE+1) = IMAT  :
% TOP(IELEM,ILNODE+2) = ITYPE :
%
% POS[ielem,ielq]   = IEQ : global equation number for local equation IELQ of
%                           element IELEM 
% DEST(inode,ildof) = IEQ : global equation number for local dof IELQ in 
%                           node INODE
%
%
%
% Where : INODE  = global node number
%         IELEM  = element number
%         ILNODE = local node number in an element
%         ILDOF  = local degree of freedom number in a node
%         ILEQ   = local equation number in an element 
%         ISD    = index of spatial dimension
%
% Modifications:
%  2-4-96: Modified algorithm to estimate maxnz



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

%**** set size parameters
[nnodes,nsd,nelem,maxnlnodes,iimat,iitype] = sysinfo(size(coord),size(top));
ndof         = max(max(dest));
%org maxnldof     = length(dest(1,:));
%org maxnz = nelem*(maxnlnodes*maxnldof)^2; 
%NEW (2-4-96):
maxnz=0;
for ielem=1:nelem
     ldof = eleminfo(ielem,coord,top,mat,1);
     maxnz=maxnz+sum(ldof)^2;
end
%NEW (2-4-96) 

%**** check input
if nargin<6
   options=[];
end
if nargin<7
   sol = [];
end
if nargin<8
   nodaldata=[];
end
if nargin<9
   elemdata_in=[];
end
if nargin<10
   elemparam=[];
end
   
if ~isempty(sol)
   nsol = length(sol(1,:));
else
   nsol = 1;
end
if ~isempty(nodaldata)
   nnoddata = length(nodaldata(1,:));
else
   nnoddata = 1;
end
if ~isempty(elemdata_in)
   nelmdata = length(elemdata_in(1,:));
else
   nelmdata = 1;
end
if isempty(options)
   options = [ 1 1 1 1 1];
end

%**** extract options
build_mass  = options(1); % if build_mass == -1 the mass matrix will be lumped
build_damp  = options(2);
build_stif  = options(3);
build_rhs   = options(4);
ielem_only  = options(5);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% Set up the element stiffness matrix and element vector and assemble
% them into the large stiffness matrix and large vector                  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
 
if build_mass>0 
  iim   = zeros(maxnz,1);
  jjm   = zeros(maxnz,1);
  mme   = zeros(maxnz,1);
  locme = zeros(nelem,1);
elseif build_mass<0  % lumped mass matrix
  mass  = zeros(ndof,1);
end

if build_damp
  iid   = zeros(maxnz,1);
  jjd   = zeros(maxnz,1);
  dde   = zeros(maxnz,1);
  locde = zeros(nelem,1);
end

if build_stif
  iis   = zeros(maxnz,1);
  jjs   = zeros(maxnz,1);
  sse   = zeros(maxnz,1);
  locse = zeros(nelem,1);
end

if build_rhs
  rhs   = zeros(ndof,1);
end

elemdata_out = zeros(nelem,1);

iposm   = 0;
iposd   = 0;
iposs   = 0;

ielemdata_in = [];
ielemparam   = [];
%**** loop over elements
for ielem=1:nelem 

  eltype = top(ielem,iitype);

  if eltype>0

    %** call element routine
    if ielem_only,
       if ~isempty(elemdata_in), ielemdata_in = elemdata_in(ielem,:); end
%       if ~isempty(elemparam),  ielemparam = elemparam(ielem,:);  end   
       [elmass,eldamp,elstif,elrhs,eldata_out] = ...
       elembld(1,coord,top(ielem,:),mat,pos(ielem,:),sol,...
               nodaldata,ielemdata_in,elemparam);
    else
       [elmass,eldamp,elstif,elrhs,eldata_out] = ...
       elembld(ielem,coord,top,mat,pos,sol,nodaldata,elemdata_in,elemparam);
    end

    %** collect element mass matrices
    if build_mass<0 & ~isempty(elmass)  

       %* lumped mass matrix, assemble as vector 
       mass = assemble(mass,lump(elmass),pos,ielem);
    
    elseif build_mass>0 & ~isempty(elmass)    
    
       %* consistent mass matrix, collect element mass matrices

       %* transform element matrix to sparse form
       [i,j,s] = find(elmass);

       %* get global equation numbers
       ii      = pos(ielem,i)'; 
       jj      = pos(ielem,j)';

       %* store equation numbers and matrix elements 
       ni=length(ii);
       iim(iposm+1:iposm+ni) = ii;
       jjm(iposm+1:iposm+ni) = jj;
       mme(iposm+1:iposm+ni) = s;

       %* store pointer (for possible later use)       
       locme(ielem) = iposm+1;

       %* update pointer
       iposm        = iposm+ni;

    end 

    %** collect element damping matrices
    if build_damp & ~isempty(eldamp)

       %* transform element matrix to sparse form
       [i,j,s] = find(eldamp);

       %* get global equation numbers
       ii      = pos(ielem,i)'; 
       jj      = pos(ielem,j)';

       %* store equation numbers and matrix elements 
       ni=length(ii);
       iid(iposd+1:iposd+ni) = ii;
       jjd(iposd+1:iposd+ni) = jj;
       dde(iposd+1:iposd+ni) = s;

       %* store pointer (for possible later use)       
       locde(ielem) = iposd+1;

       %* update pointer
       iposd        = iposd+ni;

    end
    
    %** collect element stiffness matrices
    if build_stif & ~isempty(elstif)

       %* transform element matrix to sparse form
       [i,j,s] = find(elstif);

       %* get global equation numbers
       ii      = pos(ielem,i)'; 
       jj      = pos(ielem,j)';

       %* store equation numbers and matrix elements 
       ni=length(ii);
       iis(iposs+1:iposs+ni) = ii;
       jjs(iposs+1:iposs+ni) = jj;
       sse(iposs+1:iposs+ni) = s;

       %* store pointer (for possible later use)       
       locse(ielem) = iposs+1;

       %* update pointer
       iposs        = iposs+ni;

    end
    
    %** assemble right hand side
    if build_rhs & ~isempty(elrhs)
       rhs = assemble(rhs,elrhs,pos,ielem);
    end

    %** collect element output data
    if ~isempty(eldata_out)
       eldata_out = (eldata_out(:))';
       elemdata_out(ielem,1:length(eldata_out)) = eldata_out;
    end
    
  end 

end; % loop over elements

%**** assemble global matrices
if build_mass>0 
   iim  = iim(1:iposm);
   jjm  = jjm(1:iposm);
   mme  = mme(1:iposm);
   mass = sparse(iim,jjm,mme,ndof,ndof);
   clear iim jjm mme
else
   mass=[];
end

if build_damp
   iid  = iid(1:iposd);
   jjd  = jjd(1:iposd);
   dde  = dde(1:iposd);
   damp = sparse(iid,jjd,dde,ndof,ndof);
   clear iid jjd dde
else
   damp=[];
end

if build_stif
   iis  = iis(1:iposs);
   jjs  = jjs(1:iposs);
   sse  = sse(1:iposs);
   stif = sparse(iis,jjs,sse,ndof,ndof);
   clear iis jjs sse
else
   stif = [];
end


return

