function [elnoddrv,elipdrv,elweight] = ...
   sysdrv(coord,top,mat,pos,dest,sol,nodaldata,elemdata_in,elemparam)

%  [elnoddrv,elipdrv,elweight] = ...
%    sysdrv(coord,top,mat,pos,dest,sol,nodaldata,elemdata_in,elemparam);
%
% computes derivatives of solution
%
% input:
%
% COORD       : nodal coordinates
% TOP         : connectivity structure of global nodes in the mesh
% MAT         : element/material coefficients
% POS         : global equation numbers per element  
% SOL         : solution vector
% NODALDATA   : optional nodal data array, 
%               NODALDATA(INODE,:) = data for global node INODE. Definition
%               is problem/user dependent.
% ELEMDATA_IN : optional element data array 
%               ELEMDATA_IN(IELEM,:) = data for element IELEM.  Definition
%               is problem/user dependent.
% ELEMPARAM   : optional parameter array. Definition
%               is problem/user dependent.
%
% output:
%
% elnoddrv    : derivative data in element nodal points, stored in
%               multidimensional format 
%
%               elnoddrv(IELEM,ILNODE,IVAR,ITIME) =
%               the value of derivative IVAR in local element node INODE
%               of element IELEM on timestep ITIME
%
% elipdrv     : derivative data in element integration points, stored in
%               ordinary matrix form.
%               elipdrv(IELEM,:) = data of element IELEM. 
%               The meaning of the data and its storage if problem/user
%               dependent 
% 
% ELWEIGHT    : element weighting factors for possible later averaging
%               of data on global nodal points (in multidimensional format)
%
%               ELWEIGHT(IELEM,ILNODE) =
%               weighting factor of local element node INODE
%               of element IELEM (in general the element area/volume) 

%** check input
if nargin<7
   nodaldata = [];
end
if nargin<8
   elemdata_in = [];
end
if nargin<9
   elemparam = [];
end

%** set size parameters & initialize
[nnodes,nsd,nelem,maxnlnodes,iimat,iitype] = sysinfo(size(coord),size(top));
[ndof,nsol]        = size(sol);
maxndrv = 0;
for ielem=1:nelem
    ddof    = eleminfo(ielem,coord,top,mat,2);  
    maxndrv = max([maxndrv ddof]);
end

%** initalize output arrays (some of the are multidimensional, see mdzeros)
%elnoddrv = mdzeros(nelem,maxnlnodes,maxndrv,nsol);
%elnoddrv = mdputmat(elnoddrv,NaN,[],[],[],[]);
elnoddrv = NaN*ones(nelem,maxnlnodes,maxndrv,nsol);

%elipdrv  = mdzeros(nelem,maxnlnodes,maxndrv,nsol);
%elipdrv  = mdputmat(elipdrv,NaN,[],[],[],[]);
elipdrv = NaN*ones(nelem,maxnlnodes*maxndrv*nsol);
%elweight = NaN*zeros(nelem,maxnlnodes);
%elweight = mdzeros(nelem,maxnlnodes);
%elweight = mdputmat(elweight,NaN,[],[]);
elweight = NaN*ones(nelem,maxnlnodes);

%* assemble derivative right hand side vector and nodal projection matrix 
for ielem = 1:nelem 

  eltype    = top(ielem,iitype);
  if eltype>0   %* otherwise invalid/inactive element

      %** compute element derivatives in element nodes and integration points 
      [elmdat_out1,elmdat_out2,elmdat_out3] = ...      
      elemdrv(ielem,coord,top,mat,pos,sol,nodaldata,elemdata_in,elemparam); 
      % 1: element nodal point data
      % 2: element integration point data
      % 3: element weight factors

      %** get element parameters
      ddof      = eleminfo(ielem,coord,top,mat,2);
      ndrv      = max(ddof);
      nlnodes   = length(nonzeros(top(ielem,1:maxnlnodes)));
   
      %**  store
      if ~isempty(elmdat_out1)
         %ii = mdindex(elnoddrv,ielem,1:nlnodes,1:ndrv,1:nsol);      
         %elnoddrv(ii) = elmdat_out1(:);
         elnoddrv(ielem,1:nlnodes,1:ndrv,1:nsol)=elmdat_out1;
           %* NB: note that the dimensions of elmdat_out1(:) are in the order
           %      nodes - derivative - solution, with nodes as the 
           %      first, fastest changing index
      end
      if ~isempty(elmdat_out2)
         %ii = mdindex(elipdrv,ielem,1:nlnodes,1:ndrv,1:nsol);     
         %ii = mdindex(elipdrv,ielem,[],[],[]);
         %elipdrv(ii)  = elmdat_out2(:); 
         n=length(elmdat_out2(:));
         elipdrv(ielem,1:n) = elmdat_out2(:)';
           %* NB: note that the dimensions of elmdat_out2(:) are in the order
           %      nodes - derivative - solution, with nodes as the 
           %      first, fastest changing index
      end 
      if ~isempty(elmdat_out3)
         %ii = mdindex(elweight,ielem,1:nlnodes);     
         %elweight(ii)  = elmdat_out3; 
         elweight(ielem,1:nlnodes)=elmdat_out3(:)';
      end 
      
    end

end % loop over elements



